@@ -11,14 +11,15 @@ from mwedittypes.constants import *
 def get_diff(prev_wikitext, curr_wikitext, lang='en', timeout=2, debug=False):
     """Run through full process of getting tree diff between two wikitext revisions."""
     # To provide proper structure, need all content to be nested under a section
-    if not prev_wikitext.startswith('==') and not curr_wikitext.startswith('=='):
-        prev_wikitext = '==Lede==\n' + prev_wikitext
-        curr_wikitext = '==Lede==\n' + curr_wikitext
     prev_tree = WikitextTree(wikitext=prev_wikitext, lang=lang)
     curr_tree = WikitextTree(wikitext=curr_wikitext, lang=lang)
     d = Differ(prev_tree, curr_tree, timeout=timeout)
     diff = d.get_corresponding_nodes(debug=debug)
     result = diff.post_process(prev_tree.secname_to_text, curr_tree.secname_to_text, lang=lang)
+    # this helps the node differ know that the lede is also a new section
+    # otherwise depends on headings to track changes to sections
+    if not prev_wikitext:
+        result['prev-no-content'] = True
     return result
 
 
@@ -57,9 +58,16 @@ def simple_node_class(mwnode, lang='en'):
         return nc
 
 
-def sec_to_name(mwsection, idx):
+def sec_to_name(mwsection, sidx):
     """Converts a section to an interpretible and unique name."""
-    return f'S#{idx}: {mwsection.nodes[0].title} (L{mwsection.nodes[0].level})'
+    if sidx == 0:
+        sectitle = 'Lede'
+    else:
+        try:
+            sectitle = str(mwsection.nodes[0])
+        except IndexError:
+            sectitle = ''
+    return f'{sidx}: {sectitle}'
 
 
 def node_to_name(mwnode, lang='en'):
@@ -90,10 +98,10 @@ def extract_text(mwnode, lang='en'):
         # don't collapse whitespace for tables because otherwise strip_code sometimes merges text across cells
         return mwnode.contents.strip_code(collapse=False)
     # divs/galleries almost never have true text content and can be super messy so best ignored when building Text
-    elif ntype in ('Text Formatting', 'Reference'):
-        return mwnode.contents.strip_code()
-    # Heading, Template, Comment, Argument, Category, Media, URLs without display text
-    # Tags not listed here (div, gallery, etc.)
+    elif ntype == 'Text Formatting':
+        return ''.join(extract_text(mwn) for mwn in mwnode.contents.nodes)
+    # Heading, Template, Comment, Argument, Category, Media, References, URLs without display text
+    # Tags not listed here (div, gallery, etc.) that almost never have true text content and can be super messy
     # Table elements (they duplicate the text if included)
     else:
         return ''
@@ -233,7 +241,7 @@ class WikitextTree:
         """Build tree of document nodes from Wikipedia article.
 
         This approach builds a tree with an artificial 'root' node on the 1st level,
-        all of the article sections on the 2nd level (including an artificial Lede section),
+        all of the article sections nested flatly underneath (including the Lede section),
         and all of the text, link, template, etc. nodes nested under their respective sections.
         """
         wt = mw.parse(wikitext)
@@ -249,6 +257,13 @@ class WikitextTree:
                     n_node = OrderedNode(node_to_name(n, self.lang), ntype=simple_node_class(n, self.lang), text=n,
                                          char_offset=char_offset, section=s_node.name, parent=s_node)
                     char_offset += len(str(n))
+            # empty lede
+            else:
+                sec_hash = sec_to_name(s, sidx)
+                sec_text = ''
+                self.secname_to_text[sec_hash] = sec_text
+                s_node = OrderedNode(sec_hash, ntype="Section", text=sec_text, text_hash=sec_text, char_offset=0,
+                                     section=sec_hash, parent=self.root)
 
     def expand_nested(self):
         """Expand nested nodes in tree -- e.g., Ref tags with templates/links contained in them."""
@@ -507,6 +522,7 @@ class Differ:
             for i in range(0, len(transactions)):
                 if transactions[i][0] is None:
                     ins_node = self.t2[transactions[i][1]]
+                    # this second clause allows us to later detect moved sections
                     if ins_node.ntype != 'Section' or not ins_node.children:
                         insert.append(ins_node)
                 elif transactions[i][1] is None:
@@ -699,65 +715,41 @@ class Diff:
     def _merge_text_changes(self, sections_prev, sections_curr, lang='en'):
         """Replace isolated text changes with section-level text changes."""
         changes = []
-        prev_secs_checked = set()
-        curr_secs_checked = set()
+        # check each previous section and add if not a match with its corresponding current section
+        for psec in self.sections_p_to_c:
+            csec = self.sections_p_to_c[psec]
+            if csec is None:
+                prev_text = ''.join([extract_text(n, lang) for n in mw.parse(sections_prev[psec]).nodes])
+                changes.append({'prev': {'name': node_to_name(prev_text), 'type': 'Text', 'text': prev_text,
+                                         'section': psec, 'offset': 0}})
+            elif sections_prev[psec] != sections_curr[csec]:
+                prev_text = ''.join([extract_text(n, lang) for n in mw.parse(sections_prev[psec]).nodes])
+                curr_text = ''.join([extract_text(n, lang) for n in mw.parse(sections_curr[csec]).nodes])
+                if prev_text != curr_text:
+                    changes.append({'prev': {'name': node_to_name(prev_text), 'type': 'Text', 'text': prev_text,
+                                                 'section': psec, 'offset': 0},
+                                        'curr': {'name': node_to_name(curr_text), 'type': 'Text', 'text': curr_text,
+                                                 'section': csec, 'offset': 0}})
+        # add in unmatched sections from current (new sections)
+        for csec in self.sections_c_to_p:
+            psec = self.sections_c_to_p[csec]
+            if psec is None:
+                curr_text = ''.join([extract_text(n, lang) for n in mw.parse(sections_curr[csec]).nodes])
+                changes.append({'curr': {'name': node_to_name(curr_text), 'type': 'Text', 'text': curr_text,
+                                         'section': csec, 'offset': 0}})
+
+        # remove individual text changes
         for idx in range(len(self.remove) - 1, -1, -1):
-            r = self.remove[idx]
-            if r['type'] == 'Text':
-                prev_sec = r['section']
-                if prev_sec not in prev_secs_checked:
-                    prev_secs_checked.add(prev_sec)
-                    prev_text = ''.join([extract_text(n, lang) for n in mw.parse(sections_prev[prev_sec]).nodes])
-                    curr_sec = self.sections_p_to_c[prev_sec]
-                    curr_secs_checked.add(curr_sec)
-                    if curr_sec is None:
-                        changes.append({'prev': {'name': node_to_name(prev_text), 'type': 'Text', 'text': prev_text,
-                                                 'section': prev_sec, 'offset': 0}})
-                    else:
-                        curr_text = ''.join([extract_text(n, lang) for n in mw.parse(sections_curr[curr_sec]).nodes])
-                        if prev_text != curr_text:
-                            changes.append({'prev': {'name': node_to_name(prev_text), 'type': 'Text', 'text': prev_text,
-                                                     'section': prev_sec, 'offset': 0},
-                                            'curr': {'name': node_to_name(curr_text), 'type': 'Text', 'text': curr_text,
-                                                     'section': curr_sec, 'offset': 0}})
+            if self.remove[idx]['type'] == 'Text':
                 self.remove.pop(idx)
         for idx in range(len(self.insert) - 1, -1, -1):
-            i = self.insert[idx]
-            if i['type'] == 'Text':
-                curr_sec = i['section']
-                if curr_sec not in curr_secs_checked:
-                    curr_secs_checked.add(curr_sec)
-                    curr_text = ''.join([extract_text(n, lang) for n in mw.parse(sections_curr[curr_sec]).nodes])
-                    prev_sec = self.sections_c_to_p[curr_sec]
-                    prev_secs_checked.add(prev_sec)
-                    if prev_sec is None:
-                        changes.append({'curr': {'name': node_to_name(curr_text), 'type': 'Text', 'text': curr_text,
-                                                 'section': curr_sec, 'offset': 0}})
-                    else:
-                        prev_text = ''.join([extract_text(n, lang) for n in mw.parse(sections_prev[prev_sec]).nodes])
-                        if prev_text != curr_text:
-                            changes.append({'prev': {'name': node_to_name(prev_text), 'type': 'Text', 'text': prev_text,
-                                                     'section': prev_sec, 'offset': 0},
-                                            'curr': {'name': node_to_name(curr_text), 'type': 'Text', 'text': curr_text,
-                                                     'section': curr_sec, 'offset': 0}})
+            if self.insert[idx]['type'] == 'Text':
                 self.insert.pop(idx)
         for idx in range(len(self.change) - 1, -1, -1):
-            pn = self.change[idx]['prev']
-            cn = self.change[idx]['curr']
-            if pn['type'] == 'Text':
-                prev_sec = pn['section']
-                if prev_sec not in prev_secs_checked:
-                    prev_secs_checked.add(prev_sec)
-                    prev_text = ''.join([extract_text(n, lang) for n in mw.parse(sections_prev[prev_sec]).nodes])
-                    curr_sec = cn['section']
-                    curr_text = ''.join([extract_text(n, lang) for n in mw.parse(sections_curr[curr_sec]).nodes])
-                    if prev_text != curr_text:
-                        changes.append({'prev': {'name': node_to_name(prev_text), 'type': 'Text', 'text': prev_text,
-                                                 'section': prev_sec, 'offset': 0},
-                                        'curr': {'name': node_to_name(curr_text), 'type': 'Text', 'text': curr_text,
-                                                 'section': curr_sec, 'offset': 0}})
+            if self.change[idx]['prev']['type'] == 'Text':
                 self.change.pop(idx)
 
+        # add in section-level text changes
         for c in changes:
             if 'prev' in c and 'curr' in c:
                 self.change.append(c)
